@@ -1,4 +1,4 @@
-// Copyright (c) Facebook, Inc. and its affiliates.
+// Copyright (c) Meta Platforms, Inc. and affiliates.
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 /**
@@ -12,21 +12,20 @@
 #ifndef DRGN_DEBUG_INFO_H
 #define DRGN_DEBUG_INFO_H
 
-#include <elfutils/libdw.h>
 #include <elfutils/libdwfl.h>
+#include <elfutils/version.h>
 #include <libelf.h>
 
 #include "binary_buffer.h"
 #include "cfi.h"
 #include "drgn.h"
-#include "dwarf_index.h"
+#include "dwarf_info.h"
 #include "hash_table.h"
+#include "orc_info.h"
 #include "platform.h"
 #include "string_builder.h"
 #include "vector.h"
 
-struct drgn_register_state;
-
 /**
  * @ingroup Internals
  *
@@ -34,8 +33,8 @@ struct drgn_register_state;
  *
  * Caching of debugging information.
  *
- * @ref drgn_debug_info caches debugging information (currently only DWARF). It
- * translates the debugging information to types and objects.
+ * @ref drgn_debug_info caches debugging information (currently DWARF and ORC).
+ * It translates the debugging information to types and objects.
  *
  * @{
  */
@@ -102,55 +101,16 @@ struct drgn_debug_info_module {
 	Dwfl_Module *dwfl_module;
 	struct drgn_platform platform;
 	Elf_Scn *scns[DRGN_NUM_DEBUG_SCNS];
+	Elf_Scn *alt_debug_info;
+	Elf_Scn *alt_debug_str;
 	Elf_Data *scn_data[DRGN_NUM_DEBUG_SCN_DATA];
+	Elf_Data *alt_debug_info_data;
+	Elf_Data *alt_debug_str_data;
 
-	/** Base for `DW_EH_PE_pcrel`. */
-	uint64_t pcrel_base;
-	/** Base for `DW_EH_PE_textrel`. */
-	uint64_t textrel_base;
-	/** Base for `DW_EH_PE_datarel`. */
-	uint64_t datarel_base;
-	/** Array of DWARF Common Information Entries. */
-	struct drgn_dwarf_cie *cies;
-	/**
-	 * Array of DWARF Frame Description Entries sorted by initial_location.
-	 */
-	struct drgn_dwarf_fde *fdes;
-	/** Number of elements in @ref drgn_debug_info_module::fdes. */
-	size_t num_fdes;
-
-	/**
-	 * Base for calculating program counter corresponding to an ORC unwinder
-	 * entry.
-	 *
-	 * This is the address of the `.orc_unwind_ip` ELF section.
-	 *
-	 * @sa drgn_debug_info_module::orc_entries
-	 */
-	uint64_t orc_pc_base;
-	/**
-	 * Offsets for calculating program counter corresponding to an ORC
-	 * unwinder entry.
-	 *
-	 * This is the contents of the `.orc_unwind_ip` ELF section, byte
-	 * swapped to the host's byte order if necessary.
-	 *
-	 * @sa drgn_debug_info_module::orc_entries
-	 */
-	int32_t *orc_pc_offsets;
-	/**
-	 * ORC unwinder entries.
-	 *
-	 * This is the contents of the `.orc_unwind` ELF section, byte swapped
-	 * to the host's byte order if necessary.
-	 *
-	 * Entry `i` specifies how to unwind the stack if
-	 * `orc_pc(i) <= PC < orc_pc(i + 1)`, where
-	 * `orc_pc(i) = orc_pc_base + 4 * i + orc_pc_offsets[i]`.
-	 */
-	struct drgn_orc_entry *orc_entries;
-	/** Number of ORC unwinder entries. */
-	size_t num_orc_entries;
+	/** DWARF debugging information. */
+	struct drgn_dwarf_module_info dwarf;
+	/** ORC unwinder information. */
+	struct drgn_orc_module_info orc;
 
 	/** Whether .debug_frame and .eh_frame have been parsed. */
 	bool parsed_frames;
@@ -179,6 +139,10 @@ struct drgn_debug_info_module {
 	struct drgn_debug_info_module *next;
 };
 
+struct drgn_error *
+drgn_debug_info_module_cache_section(struct drgn_debug_info_module *module,
+				     enum drgn_debug_info_scn scn);
+
 struct drgn_error *
 drgn_error_debug_info_scn(struct drgn_debug_info_module *module,
 			  enum drgn_debug_info_scn scn, const char *ptr,
@@ -207,80 +171,11 @@ drgn_debug_info_buffer_init(struct drgn_debug_info_buffer *buffer,
 	buffer->scn = scn;
 }
 
-/**
- * Find the DWARF DIEs in a @ref drgn_debug_info_module for the scope containing
- * a given program counter.
- *
- * @param[in] module Module containing @p pc.
- * @param[in] pc Program counter.
- * @param[out] bias_ret Returned difference between addresses in the loaded
- * module and addresses in the returned DIEs.
- * @param[out] dies_ret Returned DIEs. `(*dies_ret)[*length_ret - 1]` is the
- * innermost DIE containing @p pc, `(*dies_ret)[*length_ret - 2]` is its parent
- * (which may not contain @p pc itself), `(*dies_ret)[*length_ret - 3]` is its
- * grandparent, etc. Must be freed with @c free().
- * @param[out] length_ret Returned length of @p dies_ret.
- */
-struct drgn_error *
-drgn_debug_info_module_find_dwarf_scopes(struct drgn_debug_info_module *module,
-					 uint64_t pc, uint64_t *bias_ret,
-					 Dwarf_Die **dies_ret,
-					 size_t *length_ret)
-	__attribute__((__nonnull__(1, 3, 4, 5)));
-
-/**
- * Find the ancestors of a DWARF DIE.
- *
- * This finds the parent, grandparent, etc., of a DWARF DIE in the tree of DIEs.
- *
- * @param[in] module Module containing @p die.
- * @param[in] die DIE to find.
- * @param[out] dies_ret Returned DIEs. `(*dies_ret)[*length_ret]` is the DIE,
- * `(*dies_ret)[*length_ret - 1]` is its parent, `(*dies_ret)[*length_ret - 2]`
- * is its grandparent, etc., and `(*dies_ret)[0]` is the top-level unit DIE.
- * @param[out] length_ret Returned number of ancestors in @p dies_ret.
- */
-struct drgn_error *drgn_find_die_ancestors(Dwarf_Die *die, Dwarf_Die **dies_ret,
-					  size_t *length_ret)
-	__attribute__((__nonnull__(2, 3)));
-
-struct drgn_debug_info_module_key {
-	const void *build_id;
-	size_t build_id_len;
-	uint64_t start, end;
-};
-
-static inline struct drgn_debug_info_module_key
-drgn_debug_info_module_key(struct drgn_debug_info_module * const *entry)
-{
-	return (struct drgn_debug_info_module_key){
-		.build_id = (*entry)->build_id,
-		.build_id_len = (*entry)->build_id_len,
-		.start = (*entry)->start,
-		.end = (*entry)->end,
-	};
-}
 DEFINE_HASH_TABLE_TYPE(drgn_debug_info_module_table,
-		       struct drgn_debug_info_module *,
-		       drgn_debug_info_module_key)
+		       struct drgn_debug_info_module *)
 
 DEFINE_HASH_SET_TYPE(c_string_set, const char *)
 
-/** Cached type in a @ref drgn_debug_info. */
-struct drgn_dwarf_type {
-	struct drgn_type *type;
-	enum drgn_qualifiers qualifiers;
-	/**
-	 * Whether this is an incomplete array type or a typedef of one.
-	 *
-	 * This is used to work around a GCC bug; see @ref
-	 * drgn_type_from_dwarf_internal().
-	 */
-	bool is_incomplete_array;
-};
-
-DEFINE_HASH_MAP_TYPE(drgn_dwarf_type_map, const void *, struct drgn_dwarf_type);
-
 /** Cache of debugging information. */
 struct drgn_debug_info {
 	/** Program owning this cache. */
@@ -297,25 +192,8 @@ struct drgn_debug_info {
 	 * they should not be freed.
 	 */
 	struct c_string_set module_names;
-	/** Index of DWARF debugging information. */
-	struct drgn_dwarf_index dindex;
-
-	/**
-	 * Cache of parsed types.
-	 *
-	 * The key is the address of the DIE (@c Dwarf_Die::addr). The value is
-	 * a @ref drgn_dwarf_type.
-	 */
-	struct drgn_dwarf_type_map types;
-	/**
-	 * Cache of parsed types which appear to be incomplete array types but
-	 * can't be.
-	 *
-	 * See @ref drgn_type_from_dwarf_internal().
-	 */
-	struct drgn_dwarf_type_map cant_be_incomplete_array_types;
-	/** Current parsing recursion depth. */
-	int depth;
+	/** DWARF debugging information. */
+	struct drgn_dwarf_info dwarf;
 };
 
 /** Create a @ref drgn_debug_info. */
@@ -409,6 +287,14 @@ struct drgn_error *drgn_debug_info_load(struct drgn_debug_info *dbinfo,
 bool drgn_debug_info_is_indexed(struct drgn_debug_info *dbinfo,
 				const char *name);
 
+/**
+ * Get the language of the program's `main` function or `NULL` if it could not
+ * be found.
+ */
+struct drgn_error *
+drgn_debug_info_main_language(struct drgn_debug_info *dbinfo,
+			      const struct drgn_language **ret);
+
 /** @ref drgn_type_find_fn() that uses debugging information. */
 struct drgn_error *drgn_debug_info_find_type(enum drgn_type_kind kind,
 					     const char *name, size_t name_len,
@@ -422,44 +308,6 @@ drgn_debug_info_find_object(const char *name, size_t name_len,
 			    enum drgn_find_object_flags flags, void *arg,
 			    struct drgn_object *ret);
 
-/**
- * Find an object DIE in an array of DWARF scopes.
- *
- * @param[in] scopes Array of scopes, from outermost to innermost.
- * @param[in] num_scopes Number of scopes in @p scopes.
- * @param[out] die_ret Returned object DIE.
- * @param[out] type_ret If @p die_ret is a `DW_TAG_enumerator` DIE, its parent.
- * Otherwise, undefined.
- */
-struct drgn_error *drgn_find_in_dwarf_scopes(Dwarf_Die *scopes,
-					     size_t num_scopes,
-					     const char *name,
-					     Dwarf_Die *die_ret,
-					     Dwarf_Die *type_ret);
-
-/**
- * Create a @ref drgn_object from a `Dwarf_Die`.
- *
- * @param[in] die Object DIE (e.g., `DW_TAG_subprogram`, `DW_TAG_variable`,
- * `DW_TAG_formal_parameter`, `DW_TAG_enumerator`,
- * `DW_TAG_template_value_parameter`).
- * @param[in] type_die DIE of object's type. If @c NULL, use the `DW_AT_type`
- * attribute of @p die. If @p die is a `DW_TAG_enumerator` DIE, this should be
- * its parent.
- * @param[in] function_die DIE of current function. @c NULL if not in function
- * context.
- * @param[in] regs Registers of current stack frame. @c NULL if not in stack
- * frame context.
- * @param[out] ret Returned object.
- */
-struct drgn_error *
-drgn_object_from_dwarf(struct drgn_debug_info *dbinfo,
-		       struct drgn_debug_info_module *module,
-		       Dwarf_Die *die, Dwarf_Die *type_die,
-		       Dwarf_Die *function_die,
-		       const struct drgn_register_state *regs,
-		       struct drgn_object *ret);
-
 /**
  * Get the Call Frame Information in a @ref drgn_debug_info_module at a given
  * program counter.
@@ -479,12 +327,6 @@ drgn_debug_info_module_find_cfi(struct drgn_program *prog,
 				bool *interrupted_ret,
 				drgn_register_number *ret_addr_regno_ret);
 
-struct drgn_error *
-drgn_eval_cfi_dwarf_expression(struct drgn_program *prog,
-			       const struct drgn_cfi_rule *rule,
-			       const struct drgn_register_state *regs,
-			       void *buf, size_t size);
-
 struct drgn_error *open_elf_file(const char *path, int *fd_ret, Elf **elf_ret);
 
 struct drgn_error *find_elf_file(char **path_ret, int *fd_ret, Elf **elf_ret,
@@ -495,6 +337,15 @@ struct drgn_error *read_elf_section(Elf_Scn *scn, Elf_Data **ret);
 struct drgn_error *elf_address_range(Elf *elf, uint64_t bias,
 				     uint64_t *start_ret, uint64_t *end_ret);
 
+static inline Elf_Type note_header_type(uint64_t p_align)
+{
+#if _ELFUTILS_PREREQ(0, 175)
+	if (p_align == 8)
+		return ELF_T_NHDR8;
+#endif
+	return ELF_T_NHDR;
+}
+
 /** @} */
 
 #endif /* DRGN_DEBUG_INFO_H */
