@@ -1,4 +1,4 @@
-// Copyright (c) Facebook, Inc. and its affiliates.
+// Copyright (c) Meta Platforms, Inc. and affiliates.
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include <assert.h>
@@ -13,10 +13,11 @@
 #include "cfi.h"
 #include "debug_info.h"
 #include "drgn.h"
+#include "dwarf_info.h"
 #include "error.h"
-#include "hash_table.h"
 #include "helpers.h"
 #include "minmax.h"
+#include "nstring.h"
 #include "platform.h"
 #include "program.h"
 #include "register_state.h"
@@ -443,7 +444,7 @@ not_found:;
 	}
 
 	Dwarf_Die function_die = frame->scopes[frame->function_scope];
-	return drgn_object_from_dwarf(trace->prog->_dbinfo, frame->regs->module,
+	return drgn_object_from_dwarf(trace->prog->dbinfo, frame->regs->module,
 				      &die,
 				      dwarf_tag(&die) == DW_TAG_enumerator ?
 				      &type_die : NULL,
@@ -525,7 +526,7 @@ drgn_get_initial_registers(struct drgn_program *prog, uint32_t tid,
 	struct drgn_error *err;
 	struct drgn_object obj;
 	struct drgn_object tmp;
-	struct string prstatus;
+	struct nstring prstatus;
 
 	drgn_object_init(&obj, prog);
 	drgn_object_init(&tmp, prog);
