@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 import argparse
@@ -23,7 +23,7 @@ from vmtest.asynciosubprocess import (
 
 logger = logging.getLogger(__name__)
 
-KERNEL_LOCALVERSION = "-vmtest4"
+KERNEL_LOCALVERSION = "-vmtest6"
 
 
 def kconfig() -> str:
@@ -59,6 +59,8 @@ CONFIG_TMPFS=y
 CONFIG_TMPFS_XATTR=y
 CONFIG_VIRTIO_CONSOLE=y
 CONFIG_VIRTIO_PCI=y
+CONFIG_HW_RANDOM=m
+CONFIG_HW_RANDOM_VIRTIO=m
 
 # drgn needs /proc/kcore for live debugging.
 CONFIG_PROC_KCORE=y
@@ -79,12 +81,25 @@ CONFIG_KEXEC=y
 # For block tests.
 CONFIG_BLK_DEV_LOOP=m
 
+# For cgroup tests.
+CONFIG_CGROUPS=y
+
 # For kconfig tests.
 CONFIG_IKCONFIG=m
 CONFIG_IKCONFIG_PROC=y
 
 # For nodemask tests.
 CONFIG_NUMA=y
+
+# For Traffic Control tests.
+CONFIG_NET_SCHED=y
+CONFIG_NET_SCH_PRIO=m
+CONFIG_NET_SCH_SFQ=m
+CONFIG_NET_SCH_TBF=m
+CONFIG_NET_SCH_INGRESS=m
+CONFIG_NET_CLS_ACT=y
+CONFIG_NETDEVICES=y
+CONFIG_DUMMY=m
 """
 
 
